@@ -3,6 +3,7 @@
 # Distributed under the terms of the Modified BSD License.
 
 import copy
+import json
 import os
 import platform
 import time
@@ -19,8 +20,12 @@ from zmq.tests import (
 from zmq.utils.strtypes import unicode
 
 pypy = platform.python_implementation().lower() == 'pypy'
+windows = platform.platform().lower().startswith('windows')
 on_travis = bool(os.environ.get('TRAVIS_PYTHON_VERSION'))
 
+# polling on windows is slow
+POLL_TIMEOUT = 1000 if windows else 100
+
 class TestSocket(BaseZMQTestCase):
 
     def test_create(self):
@@ -231,12 +236,17 @@ class TestSocket(BaseZMQTestCase):
         a.connect(iface)
         time.sleep(0.1)
         p1 = a.send(b'something', copy=False, track=True)
-        self.assertTrue(isinstance(p1, zmq.MessageTracker))
-        self.assertFalse(p1.done)
+        assert isinstance(p1, zmq.MessageTracker)
+        assert p1 is zmq._FINISHED_TRACKER
+        # small message, should start done
+        assert p1.done
+
+        # disable zero-copy threshold
+        a.copy_threshold = 0
+
         p2 = a.send_multipart([b'something', b'else'], copy=False, track=True)
-        self.assert_(isinstance(p2, zmq.MessageTracker))
-        self.assertEqual(p2.done, False)
-        self.assertEqual(p1.done, False)
+        assert isinstance(p2, zmq.MessageTracker)
+        assert not p2.done
 
         b.bind(iface)
         msg = self.recv_multipart(b)
@@ -253,7 +263,7 @@ class TestSocket(BaseZMQTestCase):
             time.sleep(0.1)
         self.assertEqual(p2.done, True)
         self.assertEqual(msg, [b'something', b'else'])
-        m = zmq.Frame(b"again", track=True)
+        m = zmq.Frame(b"again", copy=False, track=True)
         self.assertEqual(m.tracker.done, False)
         p1 = a.send(m, copy=False)
         p2 = a.send(m, copy=False)
@@ -354,16 +364,16 @@ class TestSocket(BaseZMQTestCase):
     def test_poll(self):
         a,b = self.create_bound_pair()
         tic = time.time()
-        evt = a.poll(50)
+        evt = a.poll(POLL_TIMEOUT)
         self.assertEqual(evt, 0)
-        evt = a.poll(50, zmq.POLLOUT)
+        evt = a.poll(POLL_TIMEOUT, zmq.POLLOUT)
         self.assertEqual(evt, zmq.POLLOUT)
         msg = b'hi'
         a.send(msg)
-        evt = b.poll(50)
+        evt = b.poll(POLL_TIMEOUT)
         self.assertEqual(evt, zmq.POLLIN)
         msg2 = self.recv(b)
-        evt = b.poll(50)
+        evt = b.poll(POLL_TIMEOUT)
         self.assertEqual(evt, 0)
         self.assertEqual(msg2, msg)
     
@@ -403,7 +413,7 @@ class TestSocket(BaseZMQTestCase):
                 except AttributeError:
                     pass
             s.close()
-    
+
     def test_copy(self):
         s = self.socket(zmq.PUB)
         scopy = copy.copy(s)
@@ -413,7 +423,16 @@ class TestSocket(BaseZMQTestCase):
         self.assertEqual(s.underlying, scopy.underlying)
         self.assertEqual(s.underlying, sdcopy.underlying)
         s.close()
-    
+
+    def test_send_buffer(self):
+        a, b = self.create_bound_pair(zmq.PUSH, zmq.PULL)
+        for buffer_type in (memoryview, bytearray):
+            rawbytes = str(buffer_type).encode('ascii')
+            msg = buffer_type(rawbytes)
+            a.send(msg)
+            recvd = b.recv()
+            assert recvd == rawbytes
+
     def test_shadow(self):
         p = self.socket(zmq.PUSH)
         p.bind("tcp://127.0.0.1:5555")
@@ -428,13 +447,13 @@ class TestSocket(BaseZMQTestCase):
         p2.send(sent)
         rcvd = self.recv(s2)
         self.assertEqual(rcvd, sent)
-    
+
     def test_shadow_pyczmq(self):
         try:
             from pyczmq import zctx, zsocket
         except Exception:
             raise SkipTest("Requires pyczmq")
-        
+
         ctx = zctx.new()
         ca = zsocket.new(ctx, zmq.PUSH)
         cb = zsocket.new(ctx, zmq.PULL)
@@ -445,7 +464,7 @@ class TestSocket(BaseZMQTestCase):
         a.send(b'hi')
         rcvd = self.recv(b)
         self.assertEqual(rcvd, b'hi')
-    
+
     def test_subscribe_method(self):
         pub, sub = self.create_bound_pair(zmq.PUB, zmq.SUB)
         sub.subscribe('prefix')
@@ -466,28 +485,77 @@ class TestSocket(BaseZMQTestCase):
         pub.send(b'prefixmessage')
         events = p.poll(1000)
         self.assertEqual(events, [])
-    
+
     # Travis can't handle how much memory PyPy uses on this test
     @mark.skipif(
         (
             pypy and on_travis
         ) or (
             sys.maxsize < 2**32
+        ) or (
+            windows
         ),
         reason="only run on 64b and not on Travis."
     )
+    @mark.large
     def test_large_send(self):
+        c = os.urandom(1)
+        N = 2**31 + 1
         try:
-            buf = os.urandom(1) * (2**31 + 1)
-        except MemoryError:
-            raise SkipTest()
+            buf = c * N
+        except MemoryError as e:
+            raise SkipTest("Not enough memory: %s" % e)
         a, b = self.create_bound_pair()
-        a.send(buf, copy=False)
-        rcvd = b.recv()
-        assert rcvd == buf
+        try:
+            a.send(buf, copy=False)
+            rcvd = b.recv(copy=False)
+        except MemoryError as e:
+            raise SkipTest("Not enough memory: %s" % e)
+        # sample the front and back of the received message
+        # without checking the whole content
+        # Python 2: items in memoryview are bytes
+        # Python 3: items im memoryview are int
+        byte = c if sys.version_info < (3,) else ord(c)
+        view = memoryview(rcvd)
+        assert len(view) == N
+        assert view[0] == byte
+        assert view[-1] == byte
+
+    def test_custom_serialize(self):
+        a, b = self.create_bound_pair(zmq.DEALER, zmq.ROUTER)
+        def serialize(msg):
+            frames = []
+            frames.extend(msg.get('identities', []))
+            content = json.dumps(msg['content']).encode('utf8')
+            frames.append(content)
+            return frames
+
+        def deserialize(frames):
+            identities = frames[:-1]
+            content = json.loads(frames[-1].decode('utf8'))
+            return {
+                'identities': identities,
+                'content': content,
+            }
+        
+        msg = {
+            'content': {
+                'a': 5,
+                'b': 'bee',
+            }
+        }
+        a.send_serialized(msg, serialize)
+        recvd = b.recv_serialized(deserialize)
+        assert recvd['content'] == msg['content']
+        assert recvd['identities']
+        # bounce back, tests identities
+        b.send_serialized(recvd, serialize)
+        r2 = a.recv_serialized(deserialize)
+        assert r2['content'] == msg['content']
+        assert not r2['identities']
 
 
-if have_gevent:
+if have_gevent and not windows:
     import gevent
     
     class TestSocketGreen(GreenTest, TestSocket):
